@@ -12,6 +12,7 @@
  *  \author Peter Bell
  */
 
+// Note: do not reorder, pybind11.h must come first (it includes Python.h)
 #include <pybind11/pybind11.h>
 #include <pybind11/numpy.h>
 #include <pybind11/stl.h>
@@ -756,7 +757,7 @@ out : int
 
 } // unnamed namespace
 
-PYBIND11_MODULE(pypocketfft, m)
+PYBIND11_MODULE(pypocketfft, m, py::mod_gil_not_used())
   {
   using namespace pybind11::literals;
 
