@@ -121,5 +121,10 @@ PyInit__test_multivariate(void)
         Py_DECREF(m);
         return NULL;
     }
+
+#if Py_GIL_DISABLED
+    PyUnstable_Module_SetGIL(m, Py_MOD_GIL_NOT_USED);
+#endif
+
     return m;
 }
