@@ -238,8 +238,10 @@ class TestEstimateGradients2DGlobal:
             dz = interpnd.estimate_gradients_2d_global(tri, z, tol=1e-6)
 
             assert dz.shape == (6, 2)
-            xp_assert_close(dz, np.array(grad)[None,:] + 0*dz,
-                            rtol=1e-5, atol=1e-5, err_msg="item %d" % j)
+            xp_assert_close(
+                dz, np.array(grad)[None, :] + 0*dz, rtol=1e-5, atol=1e-5, 
+                err_msg=f"item {j}"
+            )
 
     def test_regression_2359(self):
         # Check regression --- for certain point sets, gradient
@@ -300,17 +302,22 @@ class TestCloughTocher2DInterpolator:
         ]
 
         for j, func in enumerate(funcs):
-            self._check_accuracy(func, tol=1e-13, atol=1e-7, rtol=1e-7,
-                                 err_msg="Function %d" % j)
-            self._check_accuracy(func, tol=1e-13, atol=1e-7, rtol=1e-7,
-                                 alternate=True,
-                                 err_msg="Function (alternate) %d" % j)
+            self._check_accuracy(
+                func, tol=1e-13, atol=1e-7, rtol=1e-7, err_msg=f"Function {j}"
+            )
+            self._check_accuracy(
+                func, tol=1e-13, atol=1e-7, rtol=1e-7, alternate=True, 
+                err_msg=f"Function (alternate) {j}"
+            )
             # check rescaling
-            self._check_accuracy(func, tol=1e-13, atol=1e-7, rtol=1e-7,
-                                 err_msg="Function (rescaled) %d" % j, rescale=True)
-            self._check_accuracy(func, tol=1e-13, atol=1e-7, rtol=1e-7,
-                                 alternate=True, rescale=True,
-                                 err_msg="Function (alternate, rescaled) %d" % j)
+            self._check_accuracy(
+                func, tol=1e-13, atol=1e-7, rtol=1e-7, 
+                err_msg=f"Function (rescaled) {j}", rescale=True
+            )
+            self._check_accuracy(
+                func, tol=1e-13, atol=1e-7, rtol=1e-7, alternate=True, rescale=True, 
+                err_msg=f"Function (alternate, rescaled) {j}"
+            )
 
     def test_quadratic_smoketest(self):
         # Should be reasonably accurate for quadratic functions
@@ -322,10 +329,12 @@ class TestCloughTocher2DInterpolator:
         ]
 
         for j, func in enumerate(funcs):
-            self._check_accuracy(func, tol=1e-9, atol=0.22, rtol=0,
-                                 err_msg="Function %d" % j)
-            self._check_accuracy(func, tol=1e-9, atol=0.22, rtol=0,
-                                 err_msg="Function %d" % j, rescale=True)
+            self._check_accuracy(
+                func, tol=1e-9, atol=0.22, rtol=0, err_msg=f"Function {j}"
+            )
+            self._check_accuracy(
+                func, tol=1e-9, atol=0.22, rtol=0, err_msg=f"Function {j}", rescale=True
+            )
 
     def test_tri_input(self):
         # Test at single points
@@ -379,10 +388,13 @@ class TestCloughTocher2DInterpolator:
                      rng.rand(30*30, 2)]
 
         for j, func in enumerate(funcs):
-            self._check_accuracy(func, x=grid, tol=1e-9, atol=5e-3, rtol=1e-2,
-                                 err_msg="Function %d" % j)
-            self._check_accuracy(func, x=grid, tol=1e-9, atol=5e-3, rtol=1e-2,
-                                 err_msg="Function %d" % j, rescale=True)
+            self._check_accuracy(
+                func, x=grid, tol=1e-9, atol=5e-3, rtol=1e-2, err_msg=f"Function {j}"
+            )
+            self._check_accuracy(
+                func, x=grid, tol=1e-9, atol=5e-3, rtol=1e-2, 
+                err_msg=f"Function {j}", rescale=True
+            )
 
     def test_wrong_ndim(self):
         x = np.random.randn(30, 3)
