@@ -175,5 +175,16 @@ static struct PyModuleDef _ctest = {
 PyMODINIT_FUNC
 PyInit__ctest(void)
 {
-    return PyModule_Create(&_ctest);
+    PyObject *module;
+
+    module = PyModule_Create(&_ctest);
+    if (module == NULL) {
+        return module;
+    }
+
+#if Py_GIL_DISABLED
+    PyUnstable_Module_SetGIL(module, Py_MOD_GIL_NOT_USED);
+#endif
+
+    return module;
 }
