@@ -7,52 +7,159 @@
   {"_lmder", minpack_lmder, METH_VARARGS, doc_lmder},
   {"_chkder", minpack_chkder, METH_VARARGS, doc_chkder},
  */
+#define PY_SSIZE_T_CLEAN
+#include "Python.h"
+#include "numpy/arrayobject.h"
+#include "ccallback.h"
 
-/* link libraries:
-   minpack
-   linpack_lite
-   blas
-*/
+#define PYERR(errobj,message) {PyErr_SetString(errobj,message); goto fail;}
+#define PYERR2(errobj,message) {PyErr_Print(); PyErr_SetString(errobj, message); goto fail;}
 
-/* python files:
-   minpack.py
-*/
+#define STORE_VARS() ccallback_t callback; int callback_inited = 0; jac_callback_info_t jac_callback_info;
+#define STORE_VARS_NO_INFO() ccallback_t callback; int callback_inited = 0;
 
-#if defined(NO_APPEND_FORTRAN)
-#if defined(UPPERCASE_FORTRAN)
-/* nothing to do in that case */
-#else
-#define CHKDER chkder
-#define HYBRD  hybrd
-#define HYBRJ  hybrj
-#define LMDIF  lmdif
-#define LMDER  lmder
-#define LMSTR  lmstr
-#endif
-#else
-#if defined(UPPERCASE_FORTRAN)
-#define CHKDER CHKDER_
-#define HYBRD  HYBRD_
-#define HYBRJ  HYBRJ_
-#define LMDIF  LMDIF_
-#define LMDER  LMDER_
-#define LMSTR  LMSTR_
-#else
-#define CHKDER chkder_
-#define HYBRD  hybrd_
-#define HYBRJ  hybrj_
-#define LMDIF  lmdif_
-#define LMDER  lmder_
-#define LMSTR  lmstr_
-#endif
-#endif
+#define INIT_FUNC(fun,arg,errobj) do { /* Get extra arguments or set to zero length tuple */ \
+  if (arg == NULL) { \
+    if ((arg = PyTuple_New(0)) == NULL) goto fail_free; \
+  } \
+  else \
+    Py_INCREF(arg);   /* We decrement on exit. */ \
+  if (!PyTuple_Check(arg))  \
+    PYERR(errobj,"Extra Arguments must be in a tuple"); \
+  /* Set up callback functions */ \
+  if (!PyCallable_Check(fun)) \
+    PYERR(errobj,"First argument must be a callable function."); \
+  if (init_callback(&callback, fun, arg) != 0) \
+    PYERR(errobj,"Could not init callback");\
+  callback_inited = 1; \
+  } while(0)
 
-extern void CHKDER(int*,int*,double*,double*,double*,int*,double*,double*,int*,double*);
-extern void HYBRD(void*,int*,double*,double*,double*,int*,int*,int*,double*,double*,int*,double*,int*,int*,int*,double*,int*,double*,int*,double*,double*,double*,double*,double*);
-extern void HYBRJ(void*,int*,double*,double*,double*,int*,double*,int*,double*,int*,double*,int*,int*,int*,int*,double*,int*,double*,double*,double*,double*,double*);
-extern void LMDIF(void*,int*,int*,double*,double*,double*,double*,double*,int*,double*,double*,int*,double*,int*,int*,int*,double*,int*,int*,double*,double*,double*,double*,double*);
-extern void LMDER(void*,int*,int*,double*,double*,double*,int*,double*,double*,double*,int*,double*,int*,double*,int*,int*,int*,int*,int*,double*,double*,double*,double*,double*);
-extern void LMSTR(void*,int*,int*,double*,double*,double*,int*,double*,double*,double*,int*,double*,int*,double*,int*,int*,int*,int*,int*,double*,double*,double*,double*,double*);
+#define INIT_JAC_FUNC(fun,Dfun,arg,col_deriv,errobj) do { \
+  if (arg == NULL) { \
+    if ((arg = PyTuple_New(0)) == NULL) goto fail_free; \
+  } \
+  else \
+    Py_INCREF(arg);   /* We decrement on exit. */ \
+  if (!PyTuple_Check(arg))  \
+    PYERR(errobj,"Extra Arguments must be in a tuple"); \
+  /* Set up callback functions */ \
+  if (!PyCallable_Check(fun) || (Dfun != Py_None && !PyCallable_Check(Dfun))) \
+    PYERR(errobj,"The function and its Jacobian must be callable functions."); \
+  if (init_jac_callback(&callback, &jac_callback_info, fun, Dfun, arg, col_deriv) != 0) \
+    PYERR(errobj,"Could not init callback");\
+  callback_inited = 1; \
+} while(0)
+
+#define RESTORE_JAC_FUNC() do { \
+  if (callback_inited && release_callback(&callback) != 0) { \
+    goto fail_free; \
+  }} while(0)
+
+#define RESTORE_FUNC() do { \
+  if (callback_inited && release_callback(&callback) != 0) { \
+    goto fail_free; \
+  }} while(0)
+
+#define SET_DIAG(ap_diag,o_diag,mode) { /* Set the diag vector from input */ \
+  if (o_diag == NULL || o_diag == Py_None) { \
+    ap_diag = (PyArrayObject *)PyArray_SimpleNew(1,&n,NPY_DOUBLE); \
+    if (ap_diag == NULL) goto fail; \
+    diag = (double *)PyArray_DATA(ap_diag); \
+    mode = 1; \
+  } \
+  else { \
+    ap_diag = (PyArrayObject *)PyArray_ContiguousFromObject(o_diag, NPY_DOUBLE, 1, 1); \
+    if (ap_diag == NULL) goto fail; \
+    diag = (double *)PyArray_DATA(ap_diag); \
+    mode = 2; } }
+
+#define MATRIXC2F(jac,data,m,n) {double *p1=(double *)(jac), *p2, *p3=(double *)(data);\
+int i,j;\
+for (j=0;j<(m);p3++,j++) \
+  for (p2=p3,i=0;i<(n);p2+=(m),i++,p1++) \
+    *p1 = *p2; }
+
+static PyObject *minpack_error;
+
+typedef struct {
+  PyObject *Dfun;
+  PyObject *extra_args;
+  int jac_transpose;
+} jac_callback_info_t;
+
+static PyObject *call_python_function(PyObject *func, npy_intp n, double *x, PyObject *args, int dim, PyObject *error_obj, npy_intp out_size)
+{
+  /*
+    This is a generic function to call a python function that takes a 1-D
+    sequence as a first argument and optional extra_arguments (should be a
+    zero-length tuple if none desired).  The result of the function is
+    returned in a multiarray object.
+        -- build sequence object from values in x.
+	-- add extra arguments (if any) to an argument list.
+	-- call Python callable object
+ 	-- check if error occurred:
+	         if so return NULL
+	-- if no error, place result of Python code into multiarray object.
+  */
+
+  PyArrayObject *sequence = NULL;
+  PyObject *arglist = NULL;
+  PyObject *arg1 = NULL;
+  PyObject *result = NULL;
+  PyArrayObject *result_array = NULL;
+  npy_intp fvec_sz = 0;
+
+  /* Build sequence argument from inputs */
+  sequence = (PyArrayObject *)PyArray_SimpleNewFromData(1, &n, NPY_DOUBLE, (char *)x);
+  if (sequence == NULL) PYERR2(error_obj,"Internal failure to make an array of doubles out of first\n                 argument to function call.");
+
+  /* Build argument list */
+  if ((arg1 = PyTuple_New(1)) == NULL) {
+    Py_DECREF(sequence);
+    return NULL;
+  }
+  PyTuple_SET_ITEM(arg1, 0, (PyObject *)sequence);
+                /* arg1 now owns sequence reference */
+  if ((arglist = PySequence_Concat( arg1, args)) == NULL)
+    PYERR2(error_obj,"Internal error constructing argument list.");
+
+  Py_DECREF(arg1);    /* arglist has a reference to sequence, now. */
+  arg1 = NULL;
+
+  /* Call function object --- variable passed to routine.  Extra
+          arguments are in another passed variable.
+   */
+  if ((result = PyObject_CallObject(func, arglist))==NULL) {
+      goto fail;
+  }
+
+  if ((result_array = (PyArrayObject *)PyArray_ContiguousFromObject(result, NPY_DOUBLE, dim-1, dim))==NULL)
+    PYERR2(error_obj,"Result from function call is not a proper array of floats.");
+
+  fvec_sz = PyArray_SIZE(result_array);
+  if(out_size != -1 && fvec_sz != out_size){
+      PyErr_SetString(PyExc_ValueError, "The array returned by a function changed size between calls");
+      Py_DECREF(result_array);
+      goto fail;
+  }
+
+  Py_DECREF(result);
+  Py_DECREF(arglist);
+  return (PyObject *)result_array;
+
+ fail:
+  Py_XDECREF(arglist);
+  Py_XDECREF(result);
+  Py_XDECREF(arg1);
+  return NULL;
+}
+
+void CHKDER(const int,const int,double*,const double*,const double*,const int,double*,const double*,const int,double*);
+void HYBRD(int(*)(int*,double*,double*,int*),const int,double*,double*,const double,const int,const int,const int,const double,double*,const int,const double,const int,int*,int*,double*,const int,double*,const int,double*,double*,double*,double*,double*);
+void HYBRJ(int(*)(int*,double*,double*,double*,int*,int*),const int,double*,double*,double*,const int,const double,const int,double*,const int,const double,const int,int*,int*,int*,double*,const int,double*,double*,double*,double*,double*);
+void LMDIF(int(*)(int*,int*,double*,double*,int*),const int,const int,double*,double*,const double,const double,const double,const int,const double,double*,const int,const double,const int,int*,int*,double*,const int,int*,double*,double*,double*,double*,double*);
+void LMDER(int(*)(int*,int*,double*,double*,double*,int*,int*),const int,const int,double*,double*,double*,const int,const double,const double,const double,const int,double*,const int,const double,const int,int*,int*,int*,int*,double*,double*,double*,double*,double*);
+void LMSTR(int(*)(int*,int*,double*,double*,double*,int*),const int,const int,double*,double*,double*,const int,const double,const double,const double,const int,double*,const int,const double,const int,int*,int*,int*,int*,double*,double*,double*,double*,double*);
 
 /* We only use ccallback with Python functions right now */
 static ccallback_signature_t call_signatures[] = {
@@ -68,7 +175,7 @@ static int init_callback(ccallback_t *callback, PyObject *fcn, PyObject *extra_a
   if (ret == -1) {
     return -1;
   }
-  
+
   callback->info_p = (void *)extra_args;
 
   return 0;
@@ -92,7 +199,7 @@ static int init_jac_callback(ccallback_t *callback, jac_callback_info_t *jac_cal
   jac_callback_info->Dfun = Dfun;
   jac_callback_info->extra_args = extra_args;
   jac_callback_info->jac_transpose = !col_deriv;
-  
+
   callback->info_p = (void *)jac_callback_info;
 
   return 0;
@@ -112,7 +219,7 @@ int raw_multipack_calling_function(int *n, double *x, double *fvec, int *iflag)
   PyObject *multipack_extra_arguments = (PyObject *)callback->info_p;
 
   PyArrayObject *result_array = NULL;
- 
+
   result_array = (PyArrayObject *)call_python_function(multipack_python_function, *n, x, multipack_extra_arguments, 1, minpack_error, *n);
   if (result_array == NULL) {
     *iflag = -1;
@@ -181,7 +288,7 @@ int raw_multipack_lm_function(int *m, int *n, double *x, double *fvec, int *ifla
   PyObject *multipack_extra_arguments = (PyObject *)callback->info_p;
 
   PyArrayObject *result_array = NULL;
- 
+
   result_array = (PyArrayObject *)call_python_function(multipack_python_function,*n, x, multipack_extra_arguments, 1, minpack_error, *m);
   if (result_array == NULL) {
     *iflag = -1;
@@ -225,7 +332,7 @@ int jac_multipack_lm_function(int *m, int *n, double *x, double *fvec, double *f
       *iflag = -1;
       return -1;
     }
-    if (multipack_jac_transpose == 1) 
+    if (multipack_jac_transpose == 1)
       MATRIXC2F(fjac, PyArray_DATA(result_array), *n, *ldfjac)
     else
       memcpy(fjac, PyArray_DATA(result_array), (*n)*(*ldfjac)*sizeof(double));
@@ -256,7 +363,7 @@ static PyObject *minpack_hybrd(PyObject *dummy, PyObject *args) {
   double   *wa = NULL;
 
   STORE_VARS_NO_INFO();    /* Define storage variables for global variables. */
-  
+
   if (!PyArg_ParseTuple(args, "OO|OidiiiddO", &fcn, &x0, &extra_args, &full_output, &xtol, &maxfev, &ml, &mu, &epsfcn, &factor, &o_diag)) return NULL;
 
   INIT_FUNC(fcn,extra_args,minpack_error);
@@ -303,7 +410,7 @@ static PyObject *minpack_hybrd(PyObject *dummy, PyObject *args) {
 
   /* Call the underlying FORTRAN routines. */
   n_int = n; lr_int = lr; /* cast/store/pass into HYBRD */
-  HYBRD(raw_multipack_calling_function, &n_int, x, fvec, &xtol, &maxfev, &ml, &mu, &epsfcn, diag, &mode, &factor, &nprint, &info, &nfev, fjac, &ldfjac, r, &lr_int, qtf, wa, wa+n, wa+2*n, wa+3*n);
+  HYBRD(raw_multipack_calling_function, n_int, x, fvec, xtol, maxfev, ml, mu, epsfcn, diag, mode, factor, nprint, &info, &nfev, fjac, ldfjac, r, lr_int, qtf, wa, wa+n, wa+2*n, wa+3*n);
 
   RESTORE_FUNC();
 
@@ -406,7 +513,7 @@ static PyObject *minpack_hybrj(PyObject *dummy, PyObject *args) {
 
   /* Call the underlying FORTRAN routines. */
   n_int = n; lr_int = lr; /* cast/store/pass into HYBRJ */
-  HYBRJ(jac_multipack_calling_function, &n_int, x, fvec, fjac, &ldfjac, &xtol, &maxfev, diag, &mode, &factor, &nprint, &info, &nfev, &njev, r, &lr_int, qtf, wa, wa+n, wa+2*n, wa+3*n);
+  HYBRJ(jac_multipack_calling_function, n_int, x, fvec, fjac, ldfjac, xtol, maxfev, diag, mode, factor, nprint, &info, &nfev, &njev, r, lr_int, qtf, wa, wa+n, wa+2*n, wa+3*n);
 
   RESTORE_JAC_FUNC();
 
@@ -439,7 +546,7 @@ static PyObject *minpack_hybrj(PyObject *dummy, PyObject *args) {
   Py_XDECREF(ap_qtf);
   if (allocated) free(wa);
   return NULL;
-  
+
 }
 
 /************************ Levenberg-Marquardt *******************/
@@ -507,14 +614,14 @@ static PyObject *minpack_lmdif(PyObject *dummy, PyObject *args) {
 
   /* Call the underlying FORTRAN routines. */
   n_int = n; /* to provide int*-pointed storage for int argument of LMDIF */
-  LMDIF(raw_multipack_lm_function, &m, &n_int, x, fvec, &ftol, &xtol, &gtol, &maxfev, &epsfcn, diag, &mode, &factor, &nprint, &info, &nfev, fjac, &ldfjac, ipvt, qtf, wa, wa+n, wa+2*n, wa+3*n);
-    
+  LMDIF(raw_multipack_lm_function, m, n_int, x, fvec, ftol, xtol, gtol, maxfev, epsfcn, diag, mode, factor, nprint, &info, &nfev, fjac, ldfjac, ipvt, qtf, wa, wa+n, wa+2*n, wa+3*n);
+
   RESTORE_FUNC();
 
   if (info < 0) goto fail;           /* Python error */
 
   free(wa);
-  Py_DECREF(extra_args); 
+  Py_DECREF(extra_args);
   Py_DECREF(ap_diag);
 
   if (full_output) {
@@ -539,7 +646,7 @@ static PyObject *minpack_lmdif(PyObject *dummy, PyObject *args) {
   Py_XDECREF(ap_ipvt);
   Py_XDECREF(ap_qtf);
   if (allocated) free(wa);
-  return NULL;  
+  return NULL;
 }
 
 
@@ -606,7 +713,7 @@ static PyObject *minpack_lmder(PyObject *dummy, PyObject *args) {
 
   /* Call the underlying FORTRAN routines. */
   n_int = n;
-  LMDER(jac_multipack_lm_function, &m, &n_int, x, fvec, fjac, &ldfjac, &ftol, &xtol, &gtol, &maxfev, diag, &mode, &factor, &nprint, &info, &nfev, &njev, ipvt, qtf, wa, wa+n, wa+2*n, wa+3*n);
+  LMDER(jac_multipack_lm_function, m, n_int, x, fvec, fjac, ldfjac, ftol, xtol, gtol, maxfev, diag, mode, factor, nprint, &info, &nfev, &njev, ipvt, qtf, wa, wa+n, wa+2*n, wa+3*n);
 
   RESTORE_JAC_FUNC();
 
@@ -638,7 +745,7 @@ static PyObject *minpack_lmder(PyObject *dummy, PyObject *args) {
   Py_XDECREF(ap_ipvt);
   Py_XDECREF(ap_qtf);
   if (allocated) free(wa);
-  return NULL;  
+  return NULL;
 }
 
 
@@ -671,7 +778,7 @@ static PyObject *minpack_chkder(PyObject *self, PyObject *args)
     xp = (double *)PyArray_DATA(ap_xp);
     fvecp = NULL;
     err = NULL;
-    CHKDER(&m, &n, x, fvec, fjac, &ldfjac, xp, fvecp, &mode, err);
+    CHKDER(m, n, x, fvec, fjac, ldfjac, xp, fvecp, mode, err);
   }
   else if (mode == 2) {
     if (!PyArray_IS_C_CONTIGUOUS(ap_err) || (PyArray_TYPE(ap_err) != NPY_DOUBLE))
@@ -687,13 +794,13 @@ static PyObject *minpack_chkder(PyObject *self, PyObject *args)
     fvecp = (double *)PyArray_DATA(ap_fvecp);
     err = (double *)PyArray_DATA(ap_err);
 
-    CHKDER(&m, &n, x, fvec, fjac, &m, xp, fvecp, &mode, err);
+    CHKDER(m, n, x, fvec, fjac, m, xp, fvecp, mode, err);
 
     Py_DECREF(ap_fvec);
     Py_DECREF(ap_fjac);
     Py_DECREF(ap_fvecp);
   }
-  else 
+  else
     PYERR(minpack_error,"Invalid mode, must be 1 or 2.");
 
   Py_DECREF(ap_x);
@@ -709,11 +816,56 @@ static PyObject *minpack_chkder(PyObject *self, PyObject *args)
   return NULL;
 }
 
+static struct PyMethodDef minpack_module_methods[] = {
+{"_hybrd", minpack_hybrd, METH_VARARGS, doc_hybrd},
+{"_hybrj", minpack_hybrj, METH_VARARGS, doc_hybrj},
+{"_lmdif", minpack_lmdif, METH_VARARGS, doc_lmdif},
+{"_lmder", minpack_lmder, METH_VARARGS, doc_lmder},
+{"_chkder", minpack_chkder, METH_VARARGS, doc_chkder},
+{NULL,		NULL, 0, NULL}
+};
 
+static struct PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT,
+    "_minpack",
+    NULL,
+    -1,
+    minpack_module_methods,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+};
 
+PyMODINIT_FUNC
+PyInit__minpack(void)
+{
+    PyObject *module, *mdict;
 
+    import_array();
 
+    module = PyModule_Create(&moduledef);
+    if (module == NULL) {
+        return NULL;
+    }
 
+    mdict = PyModule_GetDict(module);
+    if (mdict == NULL) {
+        return NULL;
+    }
+    minpack_error = PyErr_NewException ("_minpack.error", NULL, NULL);
+    if (minpack_error == NULL) {
+        return NULL;
+    }
+    if (PyDict_SetItemString(mdict, "error", minpack_error)) {
+        return NULL;
+    }
 
+#if Py_GIL_DISABLED
+    PyUnstable_Module_SetGIL(module, Py_MOD_GIL_NOT_USED);
+#endif
+
+    return module;
+}
 
 
