@@ -18,8 +18,8 @@ __all__ = []
 # TODO implement all relevant operations
 # use .data.__methods__() instead of /=, *=, etc.
 class _data_matrix(_spbase):
-    def __init__(self, arg1):
-        _spbase.__init__(self, arg1)
+    def __init__(self, arg1, *, maxprint=None):
+        _spbase.__init__(self, arg1, maxprint=maxprint)
 
     @property
     def dtype(self):
@@ -56,8 +56,7 @@ class _data_matrix(_spbase):
         if isscalarlike(other):
             self.data *= other
             return self
-        else:
-            return NotImplemented
+        return NotImplemented
 
     def __itruediv__(self, other):  # self /= other
         if isscalarlike(other):
@@ -97,11 +96,6 @@ class _data_matrix(_spbase):
 
     copy.__doc__ = _spbase.copy.__doc__
 
-    def count_nonzero(self):
-        return np.count_nonzero(self._deduped_data())
-
-    count_nonzero.__doc__ = _spbase.count_nonzero.__doc__
-
     def power(self, n, dtype=None):
         """
         This function performs element-wise power.
@@ -118,7 +112,7 @@ class _data_matrix(_spbase):
         ------
         NotImplementedError : if n is a zero scalar
             If zero power is desired, special case it to use
-            `np.ones(A.shape, dtype=A.dtype)`
+            ``np.ones(A.shape, dtype=A.dtype)``
         """
         if not isscalarlike(n):
             raise NotImplementedError("input is not scalar")
@@ -130,7 +124,7 @@ class _data_matrix(_spbase):
 
         data = self._deduped_data()
         if dtype is not None:
-            data = data.astype(dtype)
+            data = data.astype(dtype, copy=False)
         return self._with_data(data ** n)
 
     ###########################
@@ -177,10 +171,9 @@ class _minmax_mixin:
     These are not implemented for dia_matrix, hence the separate class.
     """
 
-    def _min_or_max_axis(self, axis, min_or_max):
+    def _min_or_max_axis(self, axis, min_or_max, explicit):
+        # already checked that self.shape[axis] is not zero
         N = self.shape[axis]
-        if N == 0:
-            raise ValueError("zero-size array to reduction operation")
         M = self.shape[1 - axis]
         idx_dtype = self._get_index_dtype(maxval=M)
 
@@ -188,11 +181,12 @@ class _minmax_mixin:
         mat.sum_duplicates()
 
         major_index, value = mat._minor_reduce(min_or_max)
-        not_full = np.diff(mat.indptr)[major_index] < N
-        value[not_full] = min_or_max(value[not_full], 0)
+        if not explicit:
+            not_full = np.diff(mat.indptr)[major_index] < N
+            value[not_full] = min_or_max(value[not_full], 0)
 
         mask = value != 0
-        major_index = np.compress(mask, major_index)
+        major_index = np.compress(mask, major_index).astype(idx_dtype, copy=False)
         value = np.compress(mask, value)
 
         if isinstance(self, sparray):
@@ -211,15 +205,11 @@ class _minmax_mixin:
                 dtype=self.dtype, shape=(M, 1)
             )
 
-    def _min_or_max(self, axis, out, min_or_max):
+    def _min_or_max(self, axis, out, min_or_max, explicit):
         if out is not None:
-            raise ValueError("Sparse arrays do not support an 'out' parameter.")
+            raise ValueError("Sparse min/max does not support an 'out' parameter.")
 
-        validateaxis(axis)
-        if self.ndim == 1:
-            if axis not in (None, 0, -1):
-                raise ValueError("axis out of range")
-            axis = None  # avoid calling special axis case. no impact on 1d
+        axis = validateaxis(axis, ndim=self.ndim)
 
         if axis is None:
             if 0 in self.shape:
@@ -229,25 +219,19 @@ class _minmax_mixin:
             if self.nnz == 0:
                 return zero
             m = min_or_max.reduce(self._deduped_data().ravel())
-            if self.nnz != math.prod(self.shape):
+            if self.nnz != math.prod(self.shape) and not explicit:
                 m = min_or_max(zero, m)
             return m
 
-        if axis < 0:
-            axis += 2
+        if any(self.shape[d] == 0 for d in axis):
+            raise ValueError("zero-size array to reduction operation")
 
-        if (axis == 0) or (axis == 1):
-            return self._min_or_max_axis(axis, min_or_max)
-        else:
-            raise ValueError("axis out of range")
-
-    def _arg_min_or_max_axis(self, axis, argmin_or_argmax, compare):
-        if self.shape[axis] == 0:
-            raise ValueError("Cannot apply the operation along a zero-sized dimension.")
-
-        if axis < 0:
-            axis += 2
+        if self.ndim == 2:
+            # note: 2D ensures that len(axis)==1 so we pass in the int axis[0]
+            return self._min_or_max_axis(axis[0], min_or_max, explicit)
+        return self._min_or_max_axis_nd(axis, min_or_max, explicit)
 
+    def _argminmax_axis(self, axis, argminmax, compare, explicit):
         zero = self.dtype.type(0)
 
         mat = self.tocsc() if axis == 0 else self.tocsr()
@@ -261,16 +245,20 @@ class _minmax_mixin:
             p, q = mat.indptr[i:i + 2]
             data = mat.data[p:q]
             indices = mat.indices[p:q]
-            extreme_index = argmin_or_argmax(data)
+            extreme_index = argminmax(data)
             extreme_value = data[extreme_index]
-            if compare(extreme_value, zero) or q - p == line_size:
-                ret[i] = indices[extreme_index]
+            if explicit:
+                if q - p > 0:
+                    ret[i] = indices[extreme_index]
             else:
-                zero_ind = _find_missing_index(indices, line_size)
-                if extreme_value == zero:
-                    ret[i] = min(extreme_index, zero_ind)
+                if compare(extreme_value, zero) or q - p == line_size:
+                    ret[i] = indices[extreme_index]
                 else:
-                    ret[i] = zero_ind
+                    zero_ind = _find_missing_index(indices, line_size)
+                    if extreme_value == zero:
+                        ret[i] = min(extreme_index, zero_ind)
+                    else:
+                        ret[i] = zero_ind
 
         if isinstance(self, sparray):
             return ret
@@ -280,58 +268,75 @@ class _minmax_mixin:
 
         return self._ascontainer(ret)
 
-    def _arg_min_or_max(self, axis, out, argmin_or_argmax, compare):
+    def _argminmax(self, axis, out, argminmax, compare, explicit):
         if out is not None:
-            raise ValueError("Sparse types do not support an 'out' parameter.")
+            minmax = "argmin" if argminmax == np.argmin else "argmax"
+            raise ValueError(f"Sparse {minmax} does not support an 'out' parameter.")
 
-        validateaxis(axis)
-
-        if self.ndim == 1:
-            if axis not in (None, 0, -1):
-                raise ValueError("axis out of range")
-            axis = None  # avoid calling special axis case. no impact on 1d
+        axis = validateaxis(axis, ndim=self.ndim)
 
         if axis is not None:
-            return self._arg_min_or_max_axis(axis, argmin_or_argmax, compare)
+            if any(self.shape[i] == 0 for i in axis):
+                minmax = "argmin" if argminmax == np.argmin else "argmax"
+                raise ValueError(f"Cannot apply {minmax} along a zero-sized dimension.")
+
+            if self.ndim == 2:
+                # note: 2D ensures that len(axis)==1 so we pass in the int axis[0]
+                return self._argminmax_axis(axis[0], argminmax, compare, explicit)
+            return self._argminmax_axis_nd(axis, argminmax, compare, explicit)
 
         if 0 in self.shape:
-            raise ValueError("Cannot apply the operation to an empty matrix.")
+            minmax = "argmin" if argminmax == np.argmin else "argmax"
+            raise ValueError(f"Cannot apply {minmax} to an empty matrix.")
 
         if self.nnz == 0:
+            if explicit:
+                minmax = "argmin" if argminmax == np.argmin else "argmax"
+                raise ValueError(f"Cannot apply {minmax} to zero matrix "
+                                 "when explicit=True.")
             return 0
 
         zero = self.dtype.type(0)
         mat = self.tocoo()
         # Convert to canonical form: no duplicates, sorted indices.
         mat.sum_duplicates()
-        extreme_index = argmin_or_argmax(mat.data)
+        extreme_index = argminmax(mat.data)
+        if explicit:
+            return extreme_index
         extreme_value = mat.data[extreme_index]
-        num_col = mat.shape[-1]
 
+        if mat.ndim > 2:
+            mat = mat.reshape(-1)
         # If the min value is less than zero, or max is greater than zero,
         # then we do not need to worry about implicit zeros.
-        if compare(extreme_value, zero):
+        # And we use a "cheap test" for the rare case of no implicit zeros.
+        maxnnz = math.prod(self.shape)
+        if compare(extreme_value, zero) or mat.nnz == maxnnz:
             # cast to Python int to avoid overflow and RuntimeError
-            return int(mat.row[extreme_index]) * num_col + int(mat.col[extreme_index])
-
-        # Cheap test for the rare case where we have no implicit zeros.
-        size = math.prod(self.shape)
-        if size == mat.nnz:
+            if mat.ndim == 1:  # includes nD case that was reshaped above
+                return int(mat.col[extreme_index])
+            # ndim == 2
+            num_col = mat.shape[-1]
             return int(mat.row[extreme_index]) * num_col + int(mat.col[extreme_index])
 
         # At this stage, any implicit zero could be the min or max value.
         # After sum_duplicates(), the `row` and `col` arrays are guaranteed to
         # be sorted in C-order, which means the linearized indices are sorted.
-        linear_indices = mat.row * num_col + mat.col
-        first_implicit_zero_index = _find_missing_index(linear_indices, size)
+        if mat.ndim == 1:  # includes nD case that was reshaped above
+            linear_indices = mat.coords[-1]
+        else:  # ndim == 2
+            num_col = mat.shape[-1]
+            linear_indices = mat.row * num_col + mat.col
+        first_implicit_zero_index = _find_missing_index(linear_indices, maxnnz)
         if extreme_value == zero:
             return min(first_implicit_zero_index, extreme_index)
         return first_implicit_zero_index
 
-    def max(self, axis=None, out=None):
-        """
-        Return the maximum of the array/matrix or maximum along an axis.
-        This takes all elements into account, not just the non-zero ones.
+    def max(self, axis=None, out=None, *, explicit=False):
+        """Return the maximum of the array/matrix or maximum along an axis.
+
+        By default, all elements are taken into account, not just the non-zero ones.
+        But with `explicit` set, only the stored elements are considered.
 
         Parameters
         ----------
@@ -345,25 +350,33 @@ class _minmax_mixin:
             compatibility reasons. Do not pass in anything except
             for the default value, as this argument is not used.
 
+        explicit : {False, True} optional (default: False)
+            When set to True, only the stored elements will be considered.
+            If a row/column is empty, the sparse.coo_array returned
+            has no stored element (i.e. an implicit zero) for that row/column.
+
+            .. versionadded:: 1.15.0
+
         Returns
         -------
-        amax : coo_matrix or scalar
+        amax : coo_array or scalar
             Maximum of `a`. If `axis` is None, the result is a scalar value.
-            If `axis` is given, the result is a sparse.coo_matrix of dimension
+            If `axis` is given, the result is a sparse.coo_array of dimension
             ``a.ndim - 1``.
 
         See Also
         --------
         min : The minimum value of a sparse array/matrix along a given axis.
-        numpy.matrix.max : NumPy's implementation of 'max' for matrices
+        numpy.max : NumPy's implementation of 'max'
 
         """
-        return self._min_or_max(axis, out, np.maximum)
+        return self._min_or_max(axis, out, np.maximum, explicit)
 
-    def min(self, axis=None, out=None):
-        """
-        Return the minimum of the array/matrix or maximum along an axis.
-        This takes all elements into account, not just the non-zero ones.
+    def min(self, axis=None, out=None, *, explicit=False):
+        """Return the minimum of the array/matrix or maximum along an axis.
+
+        By default, all elements are taken into account, not just the non-zero ones.
+        But with `explicit` set, only the stored elements are considered.
 
         Parameters
         ----------
@@ -377,26 +390,34 @@ class _minmax_mixin:
             compatibility reasons. Do not pass in anything except for
             the default value, as this argument is not used.
 
+        explicit : {False, True} optional (default: False)
+            When set to True, only the stored elements will be considered.
+            If a row/column is empty, the sparse.coo_array returned
+            has no stored element (i.e. an implicit zero) for that row/column.
+
+            .. versionadded:: 1.15.0
+
         Returns
         -------
         amin : coo_matrix or scalar
             Minimum of `a`. If `axis` is None, the result is a scalar value.
-            If `axis` is given, the result is a sparse.coo_matrix of dimension
+            If `axis` is given, the result is a sparse.coo_array of dimension
             ``a.ndim - 1``.
 
         See Also
         --------
         max : The maximum value of a sparse array/matrix along a given axis.
-        numpy.matrix.min : NumPy's implementation of 'min' for matrices
+        numpy.min : NumPy's implementation of 'min'
 
         """
-        return self._min_or_max(axis, out, np.minimum)
+        return self._min_or_max(axis, out, np.minimum, explicit)
 
-    def nanmax(self, axis=None, out=None):
-        """
-        Return the maximum of the array/matrix or maximum along an axis, ignoring any
-        NaNs. This takes all elements into account, not just the non-zero
-        ones.
+    def nanmax(self, axis=None, out=None, *, explicit=False):
+        """Return the maximum, ignoring any Nans, along an axis.
+
+        Return the maximum, ignoring any Nans, of the array/matrix along an axis.
+        By default this takes all elements into account, but with `explicit` set,
+        only stored elements are considered.
 
         .. versionadded:: 1.11.0
 
@@ -412,11 +433,18 @@ class _minmax_mixin:
             compatibility reasons. Do not pass in anything except
             for the default value, as this argument is not used.
 
+        explicit : {False, True} optional (default: False)
+            When set to True, only the stored elements will be considered.
+            If a row/column is empty, the sparse.coo_array returned
+            has no stored element (i.e. an implicit zero) for that row/column.
+
+            .. versionadded:: 1.15.0
+
         Returns
         -------
-        amax : coo_matrix or scalar
+        amax : coo_array or scalar
             Maximum of `a`. If `axis` is None, the result is a scalar value.
-            If `axis` is given, the result is a sparse.coo_matrix of dimension
+            If `axis` is given, the result is a sparse.coo_array of dimension
             ``a.ndim - 1``.
 
         See Also
@@ -428,13 +456,14 @@ class _minmax_mixin:
         numpy.nanmax : NumPy's implementation of 'nanmax'.
 
         """
-        return self._min_or_max(axis, out, np.fmax)
+        return self._min_or_max(axis, out, np.fmax, explicit)
 
-    def nanmin(self, axis=None, out=None):
-        """
-        Return the minimum of the array/matrix or minimum along an axis, ignoring any
-        NaNs. This takes all elements into account, not just the non-zero
-        ones.
+    def nanmin(self, axis=None, out=None, *, explicit=False):
+        """Return the minimum, ignoring any Nans, along an axis.
+
+        Return the minimum, ignoring any Nans, of the array/matrix along an axis.
+        By default this takes all elements into account, but with `explicit` set,
+        only stored elements are considered.
 
         .. versionadded:: 1.11.0
 
@@ -450,11 +479,18 @@ class _minmax_mixin:
             compatibility reasons. Do not pass in anything except for
             the default value, as this argument is not used.
 
+        explicit : {False, True} optional (default: False)
+            When set to True, only the stored elements will be considered.
+            If a row/column is empty, the sparse.coo_array returned
+            has no stored element (i.e. an implicit zero) for that row/column.
+
+            .. versionadded:: 1.15.0
+
         Returns
         -------
-        amin : coo_matrix or scalar
+        amin : coo_array or scalar
             Minimum of `a`. If `axis` is None, the result is a scalar value.
-            If `axis` is given, the result is a sparse.coo_matrix of dimension
+            If `axis` is given, the result is a sparse.coo_array of dimension
             ``a.ndim - 1``.
 
         See Also
@@ -466,50 +502,68 @@ class _minmax_mixin:
         numpy.nanmin : NumPy's implementation of 'nanmin'.
 
         """
-        return self._min_or_max(axis, out, np.fmin)
+        return self._min_or_max(axis, out, np.fmin, explicit)
 
-    def argmax(self, axis=None, out=None):
+    def argmax(self, axis=None, out=None, *, explicit=False):
         """Return indices of maximum elements along an axis.
 
-        Implicit zero elements are also taken into account. If there are
-        several maximum values, the index of the first occurrence is returned.
+        By default, implicit zero elements are taken into account. If there are
+        several minimum values, the index of the first occurrence is returned.
+        If `explicit` is set, only explicitly stored elements will be considered.
 
         Parameters
         ----------
         axis : {-2, -1, 0, 1, None}, optional
             Axis along which the argmax is computed. If None (default), index
             of the maximum element in the flatten data is returned.
+
         out : None, optional
             This argument is in the signature *solely* for NumPy
             compatibility reasons. Do not pass in anything except for
             the default value, as this argument is not used.
 
+        explicit : {False, True} optional (default: False)
+            When set to True, only explicitly stored elements will be considered.
+            If axis is not None and an axis has no stored elements, argmax
+            is undefined, so the index ``0`` is returned for that row/column.
+
+            .. versionadded:: 1.15.0
+
         Returns
         -------
         ind : numpy.matrix or int
             Indices of maximum elements. If matrix, its size along `axis` is 1.
         """
-        return self._arg_min_or_max(axis, out, np.argmax, np.greater)
+        return self._argminmax(axis, out, np.argmax, np.greater, explicit)
 
-    def argmin(self, axis=None, out=None):
+    def argmin(self, axis=None, out=None, *, explicit=False):
         """Return indices of minimum elements along an axis.
 
-        Implicit zero elements are also taken into account. If there are
+        By default, implicit zero elements are taken into account. If there are
         several minimum values, the index of the first occurrence is returned.
+        If `explicit` is set, only explicitly stored elements will be considered.
 
         Parameters
         ----------
         axis : {-2, -1, 0, 1, None}, optional
             Axis along which the argmin is computed. If None (default), index
             of the minimum element in the flatten data is returned.
+
         out : None, optional
             This argument is in the signature *solely* for NumPy
             compatibility reasons. Do not pass in anything except for
             the default value, as this argument is not used.
 
+        explicit : {False, True} optional (default: False)
+            When set to True, only explicitly stored elements will be considered.
+            If axis is not None and an axis has no stored elements, argmin
+            is undefined, so the index ``0`` is returned for that row/column.
+
+            .. versionadded:: 1.15.0
+
         Returns
         -------
          ind : numpy.matrix or int
             Indices of minimum elements. If matrix, its size along `axis` is 1.
         """
-        return self._arg_min_or_max(axis, out, np.argmin, np.less)
+        return self._argminmax(axis, out, np.argmin, np.less, explicit)
