@@ -1,9 +1,9 @@
 /*! \file
 Copyright (c) 2003, The Regents of the University of California, through
-Lawrence Berkeley National Laboratory (subject to receipt of any required
-approvals from U.S. Dept. of Energy)
+Lawrence Berkeley National Laboratory (subject to receipt of any required 
+approvals from U.S. Dept. of Energy) 
 
-All rights reserved.
+All rights reserved. 
 
 The source code is distributed under BSD license, see the file License.txt
 at the top-level directory.
@@ -29,11 +29,11 @@ at the top-level directory.
  *
  * SGSISX computes an approximate solutions of linear equations
  * A*X=B or A'*X=B, using the ILU factorization from sgsitrf().
- * An estimation of the condition number is provided.
+ * An estimation of the condition number is provided. 
  * The routine performs the following steps:
  *
  *   1. If A is stored column-wise (A->Stype = SLU_NC):
- *
+ *  
  *	1.1. If options->Equil = YES or options->RowPerm = LargeDiag_MC64, scaling
  *	     factors are computed to equilibrate the system:
  *	     options->Trans = NOTRANS:
@@ -226,7 +226,7 @@ at the top-level directory.
  *
  *         If options->RowPerm = LargeDiag_MC64, MC64 is used to scale and permute
  *            the matrix to an I-matrix, that is A is modified as follows:
- *            P*Dr*A*Dc has entries of modulus 1 on the diagonal and
+ *            P*Dr*A*Dc has entries of modulus 1 on the diagonal and 
  *            off-diagonal entries of modulus at most 1. P is a permutation
  *            obtained from MC64.
  *            If MC64 fails, sgsequ() is used to equilibrate the system,
@@ -244,11 +244,11 @@ at the top-level directory.
  *	   is already in postorder.
  *
  *	   If A->Stype = SLU_NR, column permutation vector of size A->nrow,
- *	   which describes permutation of columns of transpose(A)
+ *	   which describes permutation of columns of transpose(A) 
  *	   (rows of A) as described above.
  *
  * perm_r  (input/output) int*
- *	   If A->Stype = SLU_NC, row permutation vector of size A->nrow,
+ *	   If A->Stype = SLU_NC, row permutation vector of size A->nrow, 
  *	   which defines the permutation matrix Pr, and is determined
  *	   by MC64 first then followed by partial pivoting.
  *         perm_r[i] = j means row i of A is in position j in Pr*A.
@@ -276,7 +276,7 @@ at the top-level directory.
  *	   = 'N': No equilibration.
  *	   = 'R': Row equilibration, i.e., A was premultiplied by diag(R).
  *	   = 'C': Column equilibration, i.e., A was postmultiplied by diag(C).
- *	   = 'B': Both row and column equilibration, i.e., A was replaced
+ *	   = 'B': Both row and column equilibration, i.e., A was replaced 
  *		  by diag(R)*A*diag(C).
  *	   If options->Fact = FACTORED, equed is an input argument,
  *	   otherwise it is an output argument.
@@ -403,10 +403,10 @@ at the top-level directory.
 void
 sgsisx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
        int *etree, char *equed, float *R, float *C,
-       SuperMatrix *L, SuperMatrix *U, void *work, int lwork,
+       SuperMatrix *L, SuperMatrix *U, void *work, int_t lwork,
        SuperMatrix *B, SuperMatrix *X,
        float *recip_pivot_growth, float *rcond,
-       GlobalLU_t *Glu, mem_usage_t *mem_usage, SuperLUStat_t *stat, int *info)
+       GlobalLU_t *Glu, mem_usage_t *mem_usage, SuperLUStat_t *stat, int_t *info)
 {
 
     DNformat  *Bstore, *Xstore;
@@ -417,10 +417,9 @@ sgsisx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
     int       colequ, equil, nofact, notran, rowequ, permc_spec, mc64;
     trans_t   trant;
     char      norm[1];
-    int       i, j, info1;
-    float    amax, anorm, colcnd, rowcnd, rcmax, rcmin;
-    int       relax, panel_size;
-    float    diag_pivot_thresh;
+    int_t     i, j;
+    float    amax, anorm, bignum, smlnum, colcnd, rowcnd, rcmax, rcmin;
+    int       relax, panel_size, info1;
     double    t0;      /* temporary time */
     double    *utime;
 
@@ -450,15 +449,17 @@ sgsisx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
     } else {
 	rowequ = strncmp(equed, "R", 1)==0 || strncmp(equed, "B", 1)==0;
 	colequ = strncmp(equed, "C", 1)==0 || strncmp(equed, "B", 1)==0;
+	smlnum = smach("Safe minimum");  /* lamch_("Safe minimum"); */
+	bignum = 1. / smlnum;
     }
 
     /* Test the input parameters */
-    if (options->Fact != DOFACT && options->Fact != SamePattern &&
-	options->Fact != SamePattern_SameRowPerm &&
-	options->Fact != FACTORED &&
-	options->Trans != NOTRANS && options->Trans != TRANS &&
-	options->Trans != CONJ &&
-	options->Equil != NO && options->Equil != YES)
+    if ( (options->Fact != DOFACT && options->Fact != SamePattern &&
+	  options->Fact != SamePattern_SameRowPerm &&
+	  options->Fact != FACTORED) ||
+	 (options->Trans != NOTRANS && options->Trans != TRANS && 
+	  options->Trans != CONJ) ||
+	 (options->Equil != NO && options->Equil != YES) )
 	*info = -1;
     else if ( A->nrow != A->ncol || A->nrow < 0 ||
 	      (A->Stype != SLU_NC && A->Stype != SLU_NR) ||
@@ -469,9 +470,6 @@ sgsisx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
 	*info = -6;
     else {
 	if (rowequ) {
-      float smlnum, bignum;
-      smlnum = smach("Safe minimum");  /* lamch_("Safe minimum"); */
-      bignum = 1. / smlnum;
 	    rcmin = bignum;
 	    rcmax = 0.;
 	    for (j = 0; j < A->nrow; ++j) {
@@ -484,9 +482,6 @@ sgsisx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
 	    else rowcnd = 1.;
 	}
 	if (colequ && *info == 0) {
-      float smlnum, bignum;
-      smlnum = smach("Safe minimum");  /* lamch_("Safe minimum"); */
-      bignum = 1. / smlnum;
 	    rcmin = bignum;
 	    rcmax = 0.;
 	    for (j = 0; j < A->nrow; ++j) {
@@ -501,7 +496,7 @@ sgsisx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
 	if (*info == 0) {
 	    if ( lwork < -1 ) *info = -12;
 	    else if ( B->ncol < 0 || Bstore->lda < SUPERLU_MAX(0, A->nrow) ||
-		      B->Stype != SLU_DN || B->Dtype != SLU_S ||
+		      B->Stype != SLU_DN || B->Dtype != SLU_S || 
 		      B->Mtype != SLU_GE )
 		*info = -13;
 	    else if ( X->ncol < 0 || Xstore->lda < SUPERLU_MAX(0, A->nrow) ||
@@ -512,15 +507,14 @@ sgsisx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
 	}
     }
     if (*info != 0) {
-	i = -(*info);
-	input_error("sgsisx", &i);
+	int ii = -(*info);
+	input_error("sgsisx", &ii);
 	return;
     }
 
     /* Initialization for factor parameters */
     panel_size = sp_ienv(1);
     relax      = sp_ienv(2);
-    diag_pivot_thresh = options->DiagPivotThresh;
 
     utime = stat->utime;
 
@@ -546,14 +540,14 @@ sgsisx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
     if ( nofact ) {
 	register int i, j;
 	NCformat *Astore = AA->Store;
-	int nnz = Astore->nnz;
-	int *colptr = Astore->colptr;
-	int *rowind = Astore->rowind;
+	int_t nnz = Astore->nnz;
+	int_t *colptr = Astore->colptr;
+	int_t *rowind = Astore->rowind;
 	float *nzval = (float *)Astore->nzval;
 
 	if ( mc64 ) {
 	    t0 = SuperLU_timer_();
-	    if ((perm = intMalloc(n)) == NULL)
+	    if ((perm = int32Malloc(n)) == NULL)
 		ABORT("SUPERLU_MALLOC fails for perm[]");
 
 	    info1 = sldperm(5, n, nnz, colptr, rowind, nzval, perm, R, C);
@@ -589,7 +583,7 @@ sgsisx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
 	    utime[EQUIL] = SuperLU_timer_() - t0;
 	}
 
-	if ( !mc64 & equil ) { /* Only perform equilibration, no row perm */
+	if ( mc64==0 && equil ) { /* Only perform equilibration, no row perm */
 	    t0 = SuperLU_timer_();
 	    /* Compute row and column scalings to equilibrate the matrix A. */
 	    sgsequ(AA, R, C, &rowcnd, &colcnd, &amax, &info1);
@@ -606,11 +600,11 @@ sgsisx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
 
 
     if ( nofact ) {
-
+	
 	t0 = SuperLU_timer_();
 	/*
 	 * Gnet column permutation vector perm_c[], according to permc_spec:
-	 *   permc_spec = NATURAL:  natural ordering
+	 *   permc_spec = NATURAL:  natural ordering 
 	 *   permc_spec = MMD_AT_PLUS_A: minimum degree on structure of A'+A
 	 *   permc_spec = MMD_ATA:  minimum degree on structure of A'*A
 	 *   permc_spec = COLAMD:   approximate minimum degree column ordering
@@ -638,9 +632,9 @@ sgsisx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
 
 	if ( mc64 ) { /* Fold MC64's perm[] into perm_r[]. */
 	    NCformat *Astore = AA->Store;
-	    int nnz = Astore->nnz, *rowind = Astore->rowind;
+	    int_t nnz = Astore->nnz, *rowind = Astore->rowind;
 	    int *perm_tmp, *iperm;
-	    if ((perm_tmp = intMalloc(2*n)) == NULL)
+	    if ((perm_tmp = int32Malloc(2*n)) == NULL)
 		ABORT("SUPERLU_MALLOC fails for perm_tmp[]");
 	    iperm = perm_tmp + n;
 	    for (i = 0; i < n; ++i) perm_tmp[i] = perm_r[perm[i]];
@@ -678,7 +672,6 @@ sgsisx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
     }
 
     if ( nrhs > 0 ) { /* Solve the system */
-        float *rhs_work;
 
 	/* Scale and permute the right-hand side if equilibration
            and permutation from MC64 were performed. */
