@@ -8,16 +8,15 @@ import math
 
 import numpy as np
 from numpy import array, eye, exp, random
-from numpy.linalg import matrix_power
 from numpy.testing import (
         assert_allclose, assert_, assert_array_almost_equal, assert_equal,
         assert_array_almost_equal_nulp, suppress_warnings)
 
-from scipy.sparse import csc_matrix, SparseEfficiencyWarning
-from scipy.sparse._construct import eye as speye
+from scipy.sparse import csc_array, SparseEfficiencyWarning
+from scipy.sparse._construct import eye_array
 from scipy.sparse.linalg._matfuncs import (expm, _expm,
         ProductOperator, MatrixPowerOperator,
-        _onenorm_matrix_power_nnm)
+        _onenorm_matrix_power_nnm, matrix_power)
 from scipy.sparse._sputils import matrix
 from scipy.linalg import logm
 from scipy.special import factorial, binom
@@ -67,6 +66,20 @@ def test_onenorm_matrix_power_nnm():
             expected = np.linalg.norm(Mp, 1)
             assert_allclose(observed, expected)
 
+def test_matrix_power():
+    np.random.seed(1234)
+    row, col = np.random.randint(0, 4, size=(2, 6))
+    data = np.random.random(size=(6,))
+    Amat = csc_array((data, (row, col)), shape=(4, 4))
+    A = csc_array((data, (row, col)), shape=(4, 4))
+    Adense = A.toarray()
+    for power in (2, 5, 6):
+        Apow = matrix_power(A, power).toarray()
+        Amat_pow = matrix_power(Amat, power).toarray()
+        Adense_pow = np.linalg.matrix_power(Adense, power)
+        assert_allclose(Apow, Adense_pow)
+        assert_allclose(Apow, Amat_pow)
+
 
 class TestExpM:
     def test_zero_ndarray(self):
@@ -74,7 +87,7 @@ class TestExpM:
         assert_array_almost_equal(expm(a),[[1,0],[0,1]])
 
     def test_zero_sparse(self):
-        a = csc_matrix([[0.,0],[0,0]])
+        a = csc_array([[0.,0],[0,0]])
         assert_array_almost_equal(expm(a).toarray(),[[1,0],[0,1]])
 
     def test_zero_matrix(self):
@@ -87,15 +100,15 @@ class TestExpM:
         assert_allclose(expm([[1]]), A)
         assert_allclose(expm(matrix([[1]])), A)
         assert_allclose(expm(np.array([[1]])), A)
-        assert_allclose(expm(csc_matrix([[1]])).A, A)
+        assert_allclose(expm(csc_array([[1]])).toarray(), A)
         B = expm(np.array([[1j]]))
         assert_allclose(expm(((1j,),)), B)
         assert_allclose(expm([[1j]]), B)
         assert_allclose(expm(matrix([[1j]])), B)
-        assert_allclose(expm(csc_matrix([[1j]])).A, B)
+        assert_allclose(expm(csc_array([[1j]])).toarray(), B)
 
     def test_bidiagonal_sparse(self):
-        A = csc_matrix([
+        A = csc_array([
             [1, 3, 0],
             [0, 1, 5],
             [0, 0, 2]], dtype=float)
@@ -128,11 +141,10 @@ class TestExpM:
         # float32 and complex64 lead to errors in spsolve/UMFpack
         dtype = np.float64
         for scale in [1e-2, 1e-1, 5e-1, 1, 10]:
-            a = scale * speye(3, 3, dtype=dtype, format='csc')
+            a = scale * eye_array(3, 3, dtype=dtype, format='csc')
             e = exp(scale, dtype=dtype) * eye(3, dtype=dtype)
             with suppress_warnings() as sup:
-                sup.filter(SparseEfficiencyWarning,
-                           "Changing the sparsity structure of a csc_matrix is expensive.")
+                sup.filter(SparseEfficiencyWarning, "Changing the sparsity structure")
                 exact_onenorm = _expm(a, use_exact_onenorm=True).toarray()
                 inexact_onenorm = _expm(a, use_exact_onenorm=False).toarray()
             assert_array_almost_equal_nulp(exact_onenorm, e, nulp=100)
@@ -142,11 +154,10 @@ class TestExpM:
         # float32 and complex64 lead to errors in spsolve/UMFpack
         dtype = np.complex128
         for scale in [1e-2, 1e-1, 5e-1, 1, 10]:
-            a = scale * speye(3, 3, dtype=dtype, format='csc')
+            a = scale * eye_array(3, 3, dtype=dtype, format='csc')
             e = exp(scale) * eye(3, dtype=dtype)
             with suppress_warnings() as sup:
-                sup.filter(SparseEfficiencyWarning,
-                           "Changing the sparsity structure of a csc_matrix is expensive.")
+                sup.filter(SparseEfficiencyWarning, "Changing the sparsity structure")
                 assert_array_almost_equal_nulp(expm(a).toarray(), e, nulp=100)
 
     def test_logm_consistency(self):
@@ -176,8 +187,8 @@ class TestExpM:
                       [0, 0, 0, 0]], dtype=np.int16)
         assert_allclose(expm(Q), expm(1.0 * Q))
 
-        Q = csc_matrix(Q)
-        assert_allclose(expm(Q).A, expm(1.0 * Q).A)
+        Q = csc_array(Q)
+        assert_allclose(expm(Q).toarray(), expm(1.0 * Q).toarray())
 
     def test_triangularity_perturbation(self):
         # Experiment (1) of
@@ -459,7 +470,7 @@ class TestExpM:
         # This is Ward's example #4.
         # This is a version of the Forsythe matrix.
         # The eigenvector problem is badly conditioned.
-        # Ward's algorithm has difficulty esimating the accuracy
+        # Ward's algorithm has difficulty estimating the accuracy
         # of its results for this problem.
         #
         # Check the construction of one instance of this family of matrices.
@@ -577,5 +588,5 @@ class TestOperators:
             A = np.random.randn(n, n)
             B = np.random.randn(n, k)
             op = MatrixPowerOperator(A, p)
-            assert_allclose(op.matmat(B), matrix_power(A, p).dot(B))
-            assert_allclose(op.T.matmat(B), matrix_power(A, p).T.dot(B))
+            assert_allclose(op.matmat(B), np.linalg.matrix_power(A, p).dot(B))
+            assert_allclose(op.T.matmat(B), np.linalg.matrix_power(A, p).T.dot(B))
