@@ -3,10 +3,9 @@ import pytest
 import numpy as np
 from numpy.testing import assert_allclose, assert_equal, suppress_warnings
 
-from scipy.conftest import array_api_compatible
 from scipy._lib._util import rng_integers
-from scipy._lib._array_api import (is_numpy, xp_assert_close,
-                                   xp_assert_equal, array_namespace)
+from scipy._lib._array_api import is_numpy
+from scipy._lib._array_api_no_0d import xp_assert_close, xp_assert_equal
 from scipy import stats, special
 from scipy.optimize import root
 
@@ -72,22 +71,24 @@ def test_bootstrap_iv():
     with pytest.raises(ValueError, match=message):
         bootstrap(([1, 2, 3],), np.mean, n_resamples=0)
 
-    message = "'herring' cannot be used to seed a"
-    with pytest.raises(ValueError, match=message):
-        bootstrap(([1, 2, 3],), np.mean, random_state='herring')
+    message = "SeedSequence expects int or sequence of ints"
+    with pytest.raises(TypeError, match=message):
+        bootstrap(([1, 2, 3],), np.mean, rng='herring')
 
 
 @pytest.mark.parametrize("method", ['basic', 'percentile', 'BCa'])
 @pytest.mark.parametrize("axis", [0, 1, 2])
 def test_bootstrap_batch(method, axis):
     # for one-sample statistics, batch size shouldn't affect the result
-    np.random.seed(0)
+    rng = np.random.RandomState(0)
 
-    x = np.random.rand(10, 11, 12)
+    x = rng.rand(10, 11, 12)
+    # SPEC-007 leave one call with random_state to ensure it still works
     res1 = bootstrap((x,), np.mean, batch=None, method=method,
                      random_state=0, axis=axis, n_resamples=100)
+    rng = np.random.RandomState(0)
     res2 = bootstrap((x,), np.mean, batch=10, method=method,
-                     random_state=0, axis=axis, n_resamples=100)
+                     axis=axis, n_resamples=100, random_state=rng)
 
     assert_equal(res2.confidence_interval.low, res1.confidence_interval.low)
     assert_equal(res2.confidence_interval.high, res1.confidence_interval.high)
@@ -97,10 +98,10 @@ def test_bootstrap_batch(method, axis):
 @pytest.mark.parametrize("method", ['basic', 'percentile', 'BCa'])
 def test_bootstrap_paired(method):
     # test that `paired` works as expected
-    np.random.seed(0)
+    rng = np.random.RandomState(0)
     n = 100
-    x = np.random.rand(n)
-    y = np.random.rand(n)
+    x = rng.rand(n)
+    y = rng.rand(n)
 
     def my_statistic(x, y, axis=-1):
         return ((x-y)**2).mean(axis=axis)
@@ -113,8 +114,8 @@ def test_bootstrap_paired(method):
 
     i = np.arange(len(x))
 
-    res1 = bootstrap((i,), my_paired_statistic, random_state=0)
-    res2 = bootstrap((x, y), my_statistic, paired=True, random_state=0)
+    res1 = bootstrap((i,), my_paired_statistic, rng=0)
+    res2 = bootstrap((x, y), my_statistic, paired=True, rng=0)
 
     assert_allclose(res1.confidence_interval, res2.confidence_interval)
     assert_allclose(res1.standard_error, res2.standard_error)
@@ -128,7 +129,7 @@ def test_bootstrap_vectorized(method, axis, paired):
     # CI and standard_error of each axis-slice is the same as those of the
     # original 1d sample
 
-    np.random.seed(0)
+    rng = np.random.RandomState(0)
 
     def my_statistic(x, y, z, axis=-1):
         return x.mean(axis=axis) + y.mean(axis=axis) + z.mean(axis=axis)
@@ -136,11 +137,11 @@ def test_bootstrap_vectorized(method, axis, paired):
     shape = 10, 11, 12
     n_samples = shape[axis]
 
-    x = np.random.rand(n_samples)
-    y = np.random.rand(n_samples)
-    z = np.random.rand(n_samples)
+    x = rng.rand(n_samples)
+    y = rng.rand(n_samples)
+    z = rng.rand(n_samples)
     res1 = bootstrap((x, y, z), my_statistic, paired=paired, method=method,
-                     random_state=0, axis=0, n_resamples=100)
+                     rng=0, axis=0, n_resamples=100)
     assert (res1.bootstrap_distribution.shape
             == res1.standard_error.shape + (100,))
 
@@ -150,7 +151,7 @@ def test_bootstrap_vectorized(method, axis, paired):
     y = np.broadcast_to(y.reshape(reshape), shape)
     z = np.broadcast_to(z.reshape(reshape), shape)
     res2 = bootstrap((x, y, z), my_statistic, paired=paired, method=method,
-                     random_state=0, axis=axis, n_resamples=100)
+                     rng=0, axis=axis, n_resamples=100)
 
     assert_allclose(res2.confidence_interval.low,
                     res1.confidence_interval.low)
@@ -179,7 +180,7 @@ def test_bootstrap_against_theory(method):
     expected_se = dist.std()
 
     config = dict(data=(data,), statistic=np.mean, n_resamples=5000,
-                  method=method, random_state=rng)
+                  method=method, rng=rng)
     res = bootstrap(**config, confidence_level=alpha)
     assert_allclose(res.confidence_interval, expected_interval, rtol=5e-4)
     assert_allclose(res.standard_error, expected_se, atol=3e-4)
@@ -218,15 +219,10 @@ def test_bootstrap_against_R(method, expected):
     x = np.array([10, 12, 12.5, 12.5, 13.9, 15, 21, 22,
                   23, 34, 50, 81, 89, 121, 134, 213])
     res = bootstrap((x,), np.mean, n_resamples=1000000, method=method,
-                    random_state=0)
+                    rng=0)
     assert_allclose(res.confidence_interval, expected, rtol=0.005)
 
 
-tests_against_itself_1samp = {"basic": 1780,
-                              "percentile": 1784,
-                              "BCa": 1784}
-
-
 def test_multisample_BCa_against_R():
     # Because bootstrap is stochastic, it's tricky to test against reference
     # behavior. Here, we show that SciPy's BCa CI matches R wboot's BCa CI
@@ -249,11 +245,11 @@ def test_multisample_BCa_against_R():
     rng = np.random.default_rng(468865032284792692)
 
     res_basic = stats.bootstrap((x, y), statistic, method='basic',
-                                batch=100, random_state=rng)
+                                batch=100, rng=rng)
     res_percent = stats.bootstrap((x, y), statistic, method='percentile',
-                                  batch=100, random_state=rng)
+                                  batch=100, rng=rng)
     res_bca = stats.bootstrap((x, y), statistic, method='bca',
-                              batch=100, random_state=rng)
+                              batch=100, rng=rng)
 
     # compute midpoints so we can compare just one number for each
     mid_basic = np.mean(res_basic.confidence_interval)
@@ -288,7 +284,7 @@ def test_multisample_BCa_against_R():
 def test_BCa_acceleration_against_reference():
     # Compare the (deterministic) acceleration parameter for a multi-sample
     # problem against a reference value. The example is from [1], but Efron's
-    # value seems inaccurate. Straightorward code for computing the
+    # value seems inaccurate. Straightforward code for computing the
     # reference acceleration (0.011008228344026734) is available at:
     # https://github.com/scipy/scipy/pull/16455#issuecomment-1193400981
 
@@ -310,6 +306,11 @@ def test_BCa_acceleration_against_reference():
     assert_allclose(a_hat, 0.011008228344026734)
 
 
+tests_against_itself_1samp = {"basic": 1789,
+                              "percentile": 1790,
+                              "BCa": 1789}
+
+
 @pytest.mark.slow
 @pytest.mark.parametrize("method, expected",
                          tests_against_itself_1samp.items())
@@ -318,7 +319,7 @@ def test_bootstrap_against_itself_1samp(method, expected):
     # to check for unintended changes in behavior. The test also makes sure
     # that bootstrap works with multi-sample statistics and that the
     # `axis` argument works as expected / function is vectorized.
-    np.random.seed(0)
+    rng = np.random.default_rng(9123847)
 
     n = 100  # size of sample
     n_resamples = 999  # number of bootstrap resamples used to form each CI
@@ -330,14 +331,15 @@ def test_bootstrap_against_itself_1samp(method, expected):
 
     # Do the same thing 2000 times. (The code is fully vectorized.)
     n_replications = 2000
-    data = dist.rvs(size=(n_replications, n))
+    data = dist.rvs(size=(n_replications, n), random_state=rng)
     res = bootstrap((data,),
                     statistic=np.mean,
                     confidence_level=confidence_level,
                     n_resamples=n_resamples,
                     batch=50,
                     method=method,
-                    axis=-1)
+                    axis=-1,
+                    rng=rng)
     ci = res.confidence_interval
 
     # ci contains vectors of lower and upper confidence interval bounds
@@ -362,7 +364,7 @@ def test_bootstrap_against_itself_2samp(method, expected):
     # to check for unintended changes in behavior. The test also makes sure
     # that bootstrap works with multi-sample statistics and that the
     # `axis` argument works as expected / function is vectorized.
-    np.random.seed(0)
+    rng = np.random.RandomState(0)
 
     n1 = 100  # size of sample 1
     n2 = 120  # size of sample 2
@@ -382,15 +384,16 @@ def test_bootstrap_against_itself_2samp(method, expected):
 
     # Do the same thing 1000 times. (The code is fully vectorized.)
     n_replications = 1000
-    data1 = dist1.rvs(size=(n_replications, n1))
-    data2 = dist2.rvs(size=(n_replications, n2))
+    data1 = dist1.rvs(size=(n_replications, n1), random_state=rng)
+    data2 = dist2.rvs(size=(n_replications, n2), random_state=rng)
     res = bootstrap((data1, data2),
                     statistic=my_stat,
                     confidence_level=confidence_level,
                     n_resamples=n_resamples,
                     batch=50,
                     method=method,
-                    axis=-1)
+                    axis=-1,
+                    random_state=rng)
     ci = res.confidence_interval
 
     # ci contains vectors of lower and upper confidence interval bounds
@@ -416,14 +419,14 @@ def test_bootstrap_vectorized_3samp(method, axis):
             assert sample.ndim == 1
         return statistic(*data, axis=0)
 
-    np.random.seed(0)
-    x = np.random.rand(4, 5)
-    y = np.random.rand(4, 5)
-    z = np.random.rand(4, 5)
+    rng = np.random.RandomState(0)
+    x = rng.rand(4, 5)
+    y = rng.rand(4, 5)
+    z = rng.rand(4, 5)
     res1 = bootstrap((x, y, z), statistic, vectorized=True,
-                     axis=axis, n_resamples=100, method=method, random_state=0)
+                     axis=axis, n_resamples=100, method=method, rng=0)
     res2 = bootstrap((x, y, z), statistic_1d, vectorized=False,
-                     axis=axis, n_resamples=100, method=method, random_state=0)
+                     axis=axis, n_resamples=100, method=method, rng=0)
     assert_allclose(res1.confidence_interval, res2.confidence_interval)
     assert_allclose(res1.standard_error, res2.standard_error)
 
@@ -441,14 +444,14 @@ def test_bootstrap_vectorized_1samp(method, axis):
         assert x.ndim == 1
         return statistic(x, axis=0)
 
-    np.random.seed(0)
-    x = np.random.rand(4, 5)
+    rng = np.random.RandomState(0)
+    x = rng.rand(4, 5)
     res1 = bootstrap((x,), statistic, vectorized=True, axis=axis,
                      n_resamples=100, batch=None, method=method,
-                     random_state=0)
+                     rng=0)
     res2 = bootstrap((x,), statistic_1d, vectorized=False, axis=axis,
                      n_resamples=100, batch=10, method=method,
-                     random_state=0)
+                     rng=0)
     assert_allclose(res1.confidence_interval, res2.confidence_interval)
     assert_allclose(res1.standard_error, res2.standard_error)
 
@@ -477,10 +480,10 @@ def test_bootstrap_gh15678(method):
     data = dist.rvs(size=100, random_state=rng)
     data = (data,)
     res = bootstrap(data, stats.skew, method=method, n_resamples=100,
-                    random_state=np.random.default_rng(9563))
+                    rng=np.random.default_rng(9563))
     # this always worked because np.apply_along_axis returns NumPy data type
     ref = bootstrap(data, stats.skew, method=method, n_resamples=100,
-                    random_state=np.random.default_rng(9563), vectorized=False)
+                    rng=np.random.default_rng(9563), vectorized=False)
     assert_allclose(res.confidence_interval, ref.confidence_interval)
     assert_allclose(res.standard_error, ref.standard_error)
     assert isinstance(res.standard_error, np.float64)
@@ -495,10 +498,10 @@ def test_bootstrap_min():
     true_min = np.min(data)
     data = (data,)
     res = bootstrap(data, np.min, method="BCa", n_resamples=100,
-                    random_state=np.random.default_rng(3942))
+                    rng=np.random.default_rng(3942))
     assert true_min == res.confidence_interval.low
     res2 = bootstrap(-np.array(data), np.max, method="BCa", n_resamples=100,
-                     random_state=np.random.default_rng(3942))
+                     rng=np.random.default_rng(3942))
     assert_allclose(-res.confidence_interval.low,
                     res2.confidence_interval.high)
     assert_allclose(-res.confidence_interval.high,
@@ -516,14 +519,14 @@ def test_re_bootstrap(additional_resamples):
     n3 = n1 + additional_resamples
 
     rng = np.random.default_rng(296689032789913033)
-    res = stats.bootstrap((x,), np.mean, n_resamples=n1, random_state=rng,
+    res = stats.bootstrap((x,), np.mean, n_resamples=n1, rng=rng,
                           confidence_level=0.95, method='percentile')
-    res = stats.bootstrap((x,), np.mean, n_resamples=n2, random_state=rng,
+    res = stats.bootstrap((x,), np.mean, n_resamples=n2, rng=rng,
                           confidence_level=0.90, method='BCa',
                           bootstrap_result=res)
 
     rng = np.random.default_rng(296689032789913033)
-    ref = stats.bootstrap((x,), np.mean, n_resamples=n3, random_state=rng,
+    ref = stats.bootstrap((x,), np.mean, n_resamples=n3, rng=rng,
                           confidence_level=0.90, method='BCa')
 
     assert_allclose(res.standard_error, ref.standard_error, rtol=1e-14)
@@ -538,7 +541,7 @@ def test_bootstrap_alternative(method):
     dist = stats.norm(loc=2, scale=4)
     data = (dist.rvs(size=(100), random_state=rng),)
 
-    config = dict(data=data, statistic=np.std, random_state=rng, axis=-1)
+    config = dict(data=data, statistic=np.std, rng=rng, axis=-1)
     t = stats.bootstrap(**config, confidence_level=0.9)
 
     config.update(dict(n_resamples=0, bootstrap_result=t))
@@ -588,7 +591,7 @@ def test_bootstrap_resample(rng_name):
 
     np.random.seed(0)
     x = np.random.rand(*shape)
-    y = _resampling._bootstrap_resample(x, n_resamples, random_state=rng1)
+    y = _resampling._bootstrap_resample(x, n_resamples, rng=rng1)
 
     for i in range(n_resamples):
         # each resample is indexed along second to last axis
@@ -623,9 +626,9 @@ def test_percentile_along_axis():
     # _percentile_along_axis gets the q corresponding with each axis slice
 
     shape = 10, 20
-    np.random.seed(0)
-    x = np.random.rand(*shape)
-    q = np.random.rand(*shape[:-1]) * 100
+    rng = np.random.RandomState(0)
+    x = rng.rand(*shape)
+    q = rng.rand(*shape[:-1]) * 100
     y = _resampling._percentile_along_axis(x, q)
 
     for i in range(shape[0]):
@@ -651,10 +654,10 @@ def test_vectorize_statistic(axis):
     # vectorize the non-vectorized statistic
     statistic2 = _resampling._vectorize_statistic(statistic_1d)
 
-    np.random.seed(0)
-    x = np.random.rand(4, 5, 6)
-    y = np.random.rand(4, 1, 6)
-    z = np.random.rand(1, 5, 6)
+    rng = np.random.RandomState(0)
+    x = rng.rand(4, 5, 6)
+    y = rng.rand(4, 1, 6)
+    z = rng.rand(1, 5, 6)
 
     res1 = statistic(x, y, z, axis=axis)
     res2 = statistic2(x, y, z, axis=axis)
@@ -681,7 +684,7 @@ def test_vector_valued_statistic(method):
                            np.std(data, axis, ddof=1)])
 
     res = bootstrap((sample,), statistic, method=method, axis=-1,
-                    n_resamples=9999, batch=200)
+                    n_resamples=9999, batch=200, random_state=rng)
 
     counts = np.sum((res.confidence_interval.low.T < params)
                     & (res.confidence_interval.high.T > params),
@@ -724,8 +727,8 @@ def test_vector_valued_statistic_gh17715():
             [0, 8, 1, 0]]
     data = np.array(data).T
 
-    res = bootstrap(data, statistic_extradim, random_state=rng, paired=True)
-    ref = bootstrap(data, statistic, random_state=rng, paired=True)
+    res = bootstrap(data, statistic_extradim, rng=rng, paired=True)
+    ref = bootstrap(data, statistic, rng=rng, paired=True)
     assert_allclose(res.confidence_interval.low[0],
                     ref.confidence_interval.low, atol=1e-15)
     assert_allclose(res.confidence_interval.high[0],
@@ -744,13 +747,11 @@ def test_gh_20850():
     stats.bootstrap((x.T, y.T), statistic, axis=1)
     # But even when the shapes *are* the same along axis, the lengths
     # along other dimensions have to be the same (or `bootstrap` warns).
-    message = "Ignoring the dimension specified by `axis`..."
-    with pytest.warns(FutureWarning, match=message):
+    message = "Array shapes are incompatible for broadcasting."
+    with pytest.raises(ValueError, match=message):
         stats.bootstrap((x, y[:10, 0]), statistic)  # this won't work after 1.16
-    with pytest.warns(FutureWarning, match=message):
-        stats.bootstrap((x, y[:10, 0:1]), statistic)  # this will
-    with pytest.warns(FutureWarning, match=message):
-        stats.bootstrap((x.T, y.T[0:1, :10]), statistic, axis=1)  # this will
+    stats.bootstrap((x, y[:10, 0:1]), statistic)  # this will
+    stats.bootstrap((x.T, y.T[0:1, :10]), statistic, axis=1)  # this will
 
 
 # --- Test Monte Carlo Hypothesis Test --- #
@@ -771,7 +772,6 @@ class TestMonteCarloHypothesisTest:
             # return stats.ttest_1samp(x, popmean=0., axis=axis).statistic)
         return statistic
 
-    @array_api_compatible
     def test_input_validation(self, xp):
         # test that the appropriate error messages are raised for invalid input
 
@@ -838,7 +838,6 @@ class TestMonteCarloHypothesisTest:
         except ValueError as e:
             assert str(e).startswith(message)
 
-    @array_api_compatible
     def test_input_validation_xp(self, xp):
         def non_vectorized_statistic(x):
             return xp.mean(x)
@@ -855,19 +854,17 @@ class TestMonteCarloHypothesisTest:
             monte_carlo_test(sample, stats.norm.rvs, xp.mean, vectorized=False)
 
     @pytest.mark.xslow
-    @array_api_compatible
     def test_batch(self, xp):
         # make sure that the `batch` parameter is respected by checking the
         # maximum batch size provided in calls to `statistic`
         rng = np.random.default_rng(23492340193)
         x = xp.asarray(rng.standard_normal(size=10))
 
-        xp_test = array_namespace(x)  # numpy.std doesn't have `correction`
         def statistic(x, axis):
             batch_size = 1 if x.ndim == 1 else x.shape[0]
             statistic.batch_size = max(batch_size, statistic.batch_size)
             statistic.counter += 1
-            return self.get_statistic(xp_test)(x, axis=axis)
+            return self.get_statistic(xp)(x, axis=axis)
         statistic.counter = 0
         statistic.batch_size = 0
 
@@ -894,7 +891,6 @@ class TestMonteCarloHypothesisTest:
         xp_assert_equal(res1.pvalue, res3.pvalue)
         xp_assert_equal(res2.pvalue, res3.pvalue)
 
-    @array_api_compatible
     @pytest.mark.parametrize('axis', range(-3, 3))
     def test_axis_dtype(self, axis, xp):
         # test that Nd-array samples are handled correctly for valid values
@@ -917,8 +913,7 @@ class TestMonteCarloHypothesisTest:
         expected = stats.ttest_1samp(x, popmean=0., axis=axis)
 
         x = xp.asarray(x, dtype=dtype)
-        xp_test = array_namespace(x)  # numpy.std doesn't have `correction`
-        statistic = self.get_statistic(xp_test)
+        statistic = self.get_statistic(xp)
         rvs = self.get_rvs(stats.norm.rvs, rng, dtype=dtype, xp=xp)
 
         res = monte_carlo_test(x, rvs, statistic, vectorized=True,
@@ -929,7 +924,6 @@ class TestMonteCarloHypothesisTest:
         xp_assert_close(res.statistic, ref_statistic)
         xp_assert_close(res.pvalue, ref_pvalue, atol=self.atol)
 
-    @array_api_compatible
     @pytest.mark.parametrize('alternative', ("two-sided", "less", "greater"))
     def test_alternative(self, alternative, xp):
         # test that `alternative` is working as expected
@@ -939,8 +933,7 @@ class TestMonteCarloHypothesisTest:
         ref = stats.ttest_1samp(x, 0., alternative=alternative)
 
         x = xp.asarray(x)
-        xp_test = array_namespace(x)  # numpy.std doesn't have `correction`
-        statistic = self.get_statistic(xp_test)
+        statistic = self.get_statistic(xp)
         rvs = self.get_rvs(stats.norm.rvs, rng, xp=xp)
 
         res = monte_carlo_test(x, rvs, statistic, alternative=alternative)
@@ -1094,7 +1087,7 @@ class TestMonteCarloHypothesisTest:
         data = rng.random(size=(2, 5)), rng.random(size=7)  # broadcastable
         rvs = rng.normal, rng.normal
         def statistic(x, y, axis):
-            return stats.ttest_ind(x, y, axis).statistic
+            return stats.ttest_ind(x, y, axis=axis).statistic
 
         res = stats.monte_carlo_test(data, rvs, statistic, axis=-1)
         ref = stats.ttest_ind(data[0], [data[1]], axis=-1)
@@ -1118,6 +1111,7 @@ class TestMonteCarloHypothesisTest:
         assert_allclose(res.statistic, ref.statistic)
         assert_allclose(res.pvalue, ref.pvalue, atol=1e-2)
 
+    @pytest.mark.fail_slow(2)
     @pytest.mark.xfail_on_32bit("Statistic may not depend on sample order on 32-bit")
     def test_finite_precision_statistic(self):
         # Some statistics return numerically distinct values when the values
@@ -1363,12 +1357,12 @@ class TestPermutationTest:
         with pytest.raises(ValueError, match=message):
             permutation_test(([1, 2, 3], [1, 2, 3]), stat, alternative='ekki')
 
-        message = "'herring' cannot be used to seed a"
-        with pytest.raises(ValueError, match=message):
-            permutation_test(([1, 2, 3], [1, 2, 3]), stat,
-                             random_state='herring')
+        message = "SeedSequence expects int or sequence of ints"
+        with pytest.raises(TypeError, match=message):
+            permutation_test(([1, 2, 3], [1, 2, 3]), stat, rng='herring')
 
     # -- Test Parameters -- #
+    # SPEC-007 leave one call with seed to check it still works
     @pytest.mark.parametrize('random_state', [np.random.RandomState,
                                               np.random.default_rng])
     @pytest.mark.parametrize('permutation_type',
@@ -1409,6 +1403,7 @@ class TestPermutationTest:
         assert_equal(res1.pvalue, res3.pvalue)
         assert_equal(res2.pvalue, res3.pvalue)
 
+    # SPEC-007 leave at least one call with seed to check it still works
     @pytest.mark.parametrize('random_state', [np.random.RandomState,
                                               np.random.default_rng])
     @pytest.mark.parametrize('permutation_type, exact_size',
@@ -1458,7 +1453,7 @@ class TestPermutationTest:
             return np.mean(x, axis=axis) - np.mean(y, axis=axis)
 
         kwds = {'vectorized': True, 'permutation_type': 'independent',
-                'batch': 100, 'alternative': alternative, 'random_state': rng}
+                'batch': 100, 'alternative': alternative, 'rng': rng}
         res = permutation_test(data, statistic, n_resamples=permutations,
                                **kwds)
         res2 = permutation_test(data, statistic, n_resamples=np.inf, **kwds)
@@ -1484,7 +1479,7 @@ class TestPermutationTest:
             return np.mean(x - y, axis=axis)
 
         kwds = {'vectorized': True, 'permutation_type': 'samples',
-                'batch': 100, 'alternative': alternative, 'random_state': rng}
+                'batch': 100, 'alternative': alternative, 'rng': rng}
         res = permutation_test(data, statistic, n_resamples=permutations,
                                **kwds)
         res2 = permutation_test(data, statistic, n_resamples=np.inf, **kwds)
@@ -1511,7 +1506,7 @@ class TestPermutationTest:
         statistic = _resampling._vectorize_statistic(statistic1d)
 
         kwds = {'vectorized': True, 'permutation_type': 'samples',
-                'batch': 100, 'alternative': alternative, 'random_state': rng}
+                'batch': 100, 'alternative': alternative, 'rng': rng}
         res = permutation_test(data, statistic, n_resamples=permutations,
                                **kwds)
         res2 = permutation_test(data, statistic, n_resamples=np.inf, **kwds)
@@ -1519,35 +1514,6 @@ class TestPermutationTest:
         assert res.statistic == res2.statistic
         assert_allclose(res.pvalue, res2.pvalue, atol=1e-2)
 
-    @pytest.mark.parametrize('alternative', ('less', 'greater'))
-    # Different conventions for two-sided p-value here VS ttest_ind.
-    # Eventually, we can add multiple options for the two-sided alternative
-    # here in permutation_test.
-    @pytest.mark.parametrize('permutations', (30, 1e9))
-    @pytest.mark.parametrize('axis', (0, 1, 2))
-    def test_against_permutation_ttest(self, alternative, permutations, axis):
-        # check that this function and ttest_ind with permutations give
-        # essentially identical results.
-
-        x = np.arange(3*4*5).reshape(3, 4, 5)
-        y = np.moveaxis(np.arange(4)[:, None, None], 0, axis)
-
-        rng1 = np.random.default_rng(4337234444626115331)
-        res1 = stats.ttest_ind(x, y, permutations=permutations, axis=axis,
-                               random_state=rng1, alternative=alternative)
-
-        def statistic(x, y, axis):
-            return stats.ttest_ind(x, y, axis=axis).statistic
-
-        rng2 = np.random.default_rng(4337234444626115331)
-        res2 = permutation_test((x, y), statistic, vectorized=True,
-                                n_resamples=permutations,
-                                alternative=alternative, axis=axis,
-                                random_state=rng2)
-
-        assert_allclose(res1.statistic, res2.statistic, rtol=self.rtol)
-        assert_allclose(res1.pvalue, res2.pvalue, rtol=self.rtol)
-
     # -- Independent (Unpaired) Sample Tests -- #
 
     @pytest.mark.parametrize('alternative', ("less", "greater", "two-sided"))
@@ -1565,7 +1531,7 @@ class TestPermutationTest:
         # ks_2samp is always a one-tailed 'greater' test
         # it's the statistic that changes (D+ vs D- vs max(D+, D-))
         res = permutation_test((x, y), statistic1d, n_resamples=np.inf,
-                               alternative='greater', random_state=self.rng)
+                               alternative='greater', rng=self.rng)
 
         assert_allclose(res.statistic, expected.statistic, rtol=self.rtol)
         assert_allclose(res.pvalue, expected.pvalue, rtol=self.rtol)
@@ -1587,7 +1553,7 @@ class TestPermutationTest:
             return stats.ansari(x, y).statistic
 
         res = permutation_test((x, y), statistic1d, n_resamples=np.inf,
-                               alternative=alternative, random_state=self.rng)
+                               alternative=alternative, rng=self.rng)
 
         assert_allclose(res.statistic, expected.statistic, rtol=self.rtol)
         assert_allclose(res.pvalue, expected.pvalue, rtol=self.rtol)
@@ -1605,7 +1571,7 @@ class TestPermutationTest:
 
         res = permutation_test((x, y), statistic, vectorized=True,
                                n_resamples=np.inf, alternative=alternative,
-                               axis=1, random_state=self.rng)
+                               axis=1, rng=self.rng)
 
         assert_allclose(res.statistic, expected.statistic, rtol=self.rtol)
         assert_allclose(res.pvalue, expected.pvalue, rtol=self.rtol)
@@ -1623,7 +1589,7 @@ class TestPermutationTest:
 
         # cramervonmises_2samp has only one alternative, greater
         res = permutation_test((x, y), statistic1d, n_resamples=np.inf,
-                               alternative='greater', random_state=self.rng)
+                               alternative='greater', rng=self.rng)
 
         assert_allclose(res.statistic, expected.statistic, rtol=self.rtol)
         assert_allclose(res.pvalue, expected.pvalue, rtol=self.rtol)
@@ -1664,7 +1630,7 @@ class TestPermutationTest:
 
         # Calculate exact and randomized permutation results
         kwds = {'vectorized': False, 'axis': axis, 'alternative': 'greater',
-                'permutation_type': 'independent', 'random_state': self.rng}
+                'permutation_type': 'independent', 'rng': self.rng}
         res = permutation_test(data, statistic1d, n_resamples=np.inf, **kwds)
         res2 = permutation_test(data, statistic1d, n_resamples=1000, **kwds)
 
@@ -1702,7 +1668,7 @@ class TestPermutationTest:
         expected_p = expected[1]
 
         kwds = {'vectorized': False, 'axis': 1, 'alternative': alternative,
-                'permutation_type': 'samples', 'random_state': self.rng,
+                'permutation_type': 'samples', 'rng': self.rng,
                 'n_resamples': np.inf}
         res1 = permutation_test((x-y,), statistic_1samp_1d, **kwds)
         res2 = permutation_test((x, y), statistic_2samp_1d, **kwds)
@@ -1732,7 +1698,7 @@ class TestPermutationTest:
 
         res = stats.permutation_test((x,), statistic, vectorized=True,
                                      permutation_type='samples',
-                                     n_resamples=np.inf, random_state=self.rng,
+                                     n_resamples=np.inf, rng=self.rng,
                                      alternative=alternative)
         assert_allclose(res.pvalue, expected.pvalue, rtol=self.rtol)
 
@@ -1750,7 +1716,7 @@ class TestPermutationTest:
 
         # kendalltau currently has only one alternative, two-sided
         res = permutation_test((x,), statistic1d, permutation_type='pairings',
-                               n_resamples=np.inf, random_state=self.rng)
+                               n_resamples=np.inf, rng=self.rng)
 
         assert_allclose(res.statistic, expected.statistic, rtol=self.rtol)
         assert_allclose(res.pvalue, expected.pvalue, rtol=self.rtol)
@@ -1769,7 +1735,7 @@ class TestPermutationTest:
 
         res = permutation_test((x,), statistic, permutation_type='pairings',
                                n_resamples=np.inf, alternative=alternative,
-                               random_state=rng)
+                               rng=rng)
         res2 = stats.fisher_exact(tab, alternative=alternative)
 
         assert_allclose(res.pvalue, res2[1])
@@ -1808,7 +1774,7 @@ class TestPermutationTest:
 
         # Let's forgive this use of an integer seed, please.
         kwds = {'vectorized': False, 'axis': axis, 'alternative': 'greater',
-                'permutation_type': 'pairings', 'random_state': 0}
+                'permutation_type': 'pairings', 'rng': 0}
         res = permutation_test(data, statistic1d, n_resamples=np.inf, **kwds)
         res2 = permutation_test(data, statistic1d, n_resamples=5000, **kwds)
 
@@ -1945,6 +1911,7 @@ class TestPermutationTest:
         got = list(_resampling._batch_generator(iterable, batch))
         assert got == expected
 
+    @pytest.mark.fail_slow(2)
     def test_finite_precision_statistic(self):
         # Some statistics return numerically distinct values when the values
         # should be equal in theory. Test that `permutation_test` accounts
@@ -2001,9 +1968,9 @@ def test_parameter_vectorized(fun_name):
     def rvs(size):  # needed by `monte_carlo_test`
         return stats.norm.rvs(size=size, random_state=rng)
 
-    fun_options = {'bootstrap': {'data': (sample,), 'random_state': rng,
+    fun_options = {'bootstrap': {'data': (sample,), 'rng': rng,
                                  'method': 'percentile'},
-                   'permutation_test': {'data': (sample,), 'random_state': rng,
+                   'permutation_test': {'data': (sample,), 'rng': rng,
                                         'permutation_type': 'samples'},
                    'monte_carlo_test': {'sample': sample, 'rvs': rvs}}
     common_options = {'n_resamples': 100}
@@ -2023,3 +1990,11 @@ def test_parameter_vectorized(fun_name):
         return np.mean(x)
     fun(statistic=statistic, vectorized=None, **options)
     fun(statistic=statistic, vectorized=False, **options)
+
+
+class TestMonteCarloMethod:
+    def test_rvs_and_random_state(self):
+        message = "Use of `rvs` and `rng` are mutually exclusive."
+        rng = np.random.default_rng(34982345)
+        with pytest.raises(ValueError, match=message):
+            stats.MonteCarloMethod(rvs=rng.random, rng=rng)
